/*	$NetBSD: bootxx.c,v 1.10 2001/02/22 07:11:10 chs Exp $ */

/*-
 * Copyright (c) 1998 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Paul Kranenburg.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by the NetBSD
 *        Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * This is a generic "first-stage" boot program.
 *
 * Note that this program has absolutely no filesystem knowledge!
 *
 * Instead, this uses a table of disk block numbers that are
 * filled in by the installboot program such that this program
 * can load the "second-stage" boot program.
 */

#include <sys/param.h>
#include <machine/mon.h>

#include <stand.h>
#include "libsa.h"
#include "saio.h"

/*
 * This is the address where we load the second-stage boot loader.
 */
#define LOADADDR	0x4000

/* This determines the largest boot program we can load. */
#define MAXBLOCKNUM	128

/*
 * These three names are known by installboot.
 * The block_table contains starting block numbers,
 * in terms of 512-byte blocks.  Each non-zero value
 * will result in a read of block_size bytes.
 */
//int     	block_size;	/* default */
//int     	block_count;	/* length of table */
daddr_t 	block_table[MAXBLOCKNUM];

struct saioreq si;
struct bootparam *bp;
struct boottab *ops;

extern unsigned long _membegin;
extern void *chain_to_func;

static int do_strat(int bn, void *ma)
{
	si.si_bn = bn;
	si.si_ma = ma;
	si.si_cc = 512;
	
	return (*ops->b_strategy)(&si, SAIO_F_READ);
}

void
main(void)
{
	void	*entry;
	char	*addr;
	int n;
	daddr_t *block;
	daddr_t mapblk;
	
	bp = *romVectorPtr->bootParam;
	ops = si.si_boottab = bp->bootDevice;
	si.si_ctlr = bp->ctlrNum;
	si.si_unit = bp->unitNum;
	si.si_boff = bp->partNum;
	si.si_dmaaddr = 0x0ff00000;

	n = (*ops->b_open)(&si);
	if(n != 0) 
		goto failed;

	addr = (char*)LOADADDR;
	entry=addr;

	mapblk = *(daddr_t *)((unsigned long)&_membegin + 508);
	n = do_strat(mapblk, block_table);
	if(n <= 0)
		goto failed;

	for(block = block_table; *block != 0; block++, addr += 512) {
		n = do_strat(*block, addr);
		if(n <= 0)
			goto failed;
//		mon_printf("R %x\n", n);
	}
		

//	chain_to(entry);
//	(*ops->b_close)(&si);	
	chain_to_func = entry;
	ICIA();
	exit();

 failed:
	mon_printf("boot: fail\n");
	mon_exit_to_mon();
	
	while(1);

}
