/* slime - construct bootinfo records */

#define _SUN3X_

#include <sys/param.h>
#include <stand.h>
#include <lib/libkern/libkern.h>
#include <machine/mon.h>

/* From the linux kernel - for bootinfo tags */

#include <linux/bootinfo.h>
#include <linux/setup.h>

void *add_tag(void *start, unsigned short id, unsigned short size, void *data)
{
    struct bi_record *bi;

    bi = (struct bi_record *)start;
    bi->tag = id;
    bi->size = size + sizeof(bi);

    if (size)
        memcpy(&bi->data[0], data, size);

    return (void *)((char *)start + bi->size);
}

void *add_tag_ul(void *start, unsigned short id, unsigned long data)
{
    return add_tag(start, id, sizeof(data), &data);
}

void *init_bootinfo(void *start, char *params, void *rddata, int rdsize, unsigned long machtype)
{
    unsigned memavail, memtotal;
    struct physmemory *p;
    struct mem_info m;

    if(machtype) {
	    /* sun3x setup */
	    start = add_tag_ul(start, BI_MACHTYPE, MACH_SUN3X);
	    start = add_tag_ul(start, BI_CPUTYPE, CPU_68030);
	    start = add_tag_ul(start, BI_FPUTYPE, FPU_68882);
	    start = add_tag_ul(start, BI_MMUTYPE, MMU_68030);

	    /* Copy the prom memory info into bootinfo */
	    memavail = *(romVectorPtr->memoryAvail);
	    
	    p = romVectorPtr->v_physmemory;
	    memtotal = 0;
	    
	    if (!p)
		    panic("No memory chunk info available");
	    
	    while (p) {
		    if (memtotal)
			    start = add_tag(start, BI_MEMCHUNK, sizeof(m), &m);
		    
		    m.addr = p->address;
		    m.size = p->size;
		    
		    memtotal += p->size;
		    
		    p = p->next;
	    }
	    
	    /* Adjust the last bank for any space the PROM requires
	     */
	    
	    if (memtotal > memavail) {
		    unsigned adj = memtotal - memavail;
		    
		    m.size -= adj;
	    }
	    
	    start = add_tag(start, BI_MEMCHUNK, sizeof(m), &m);

    } else {
	    /* sun3 setup */
	    start = add_tag_ul(start, BI_MACHTYPE, MACH_SUN3);
	    start = add_tag_ul(start, BI_CPUTYPE, CPU_68020);
	    start = add_tag_ul(start, BI_FPUTYPE, FPU_68881);
	    start = add_tag_ul(start, BI_MMUTYPE, MMU_SUN3);
    }

    if (params) {
        char cmdline[CL_SIZE];
        strncpy(cmdline, params, CL_SIZE);
        cmdline[CL_SIZE-1] = 0;
        start = add_tag(start, BI_COMMAND_LINE, CL_SIZE, cmdline);
    }

    if (rddata) {
        struct mem_info rdinfo;

        rdinfo.addr = (unsigned long)rddata;
        rdinfo.size = rdsize;
        
        start = add_tag(start, BI_RAMDISK, sizeof(rdinfo), &rdinfo);
    }
    
    start = add_tag(start, BI_LAST, 0, NULL);

    return start;
}
